package com.jtricks.servlet;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.atlassian.crowd.embedded.api.User;
import com.atlassian.jira.component.ComponentAccessor;
import com.atlassian.jira.config.properties.APKeys;
import com.atlassian.jira.event.issue.IssueEvent;
import com.atlassian.jira.event.issue.IssueEventManager;
import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.mail.IssueMailQueueItemFactory;
import com.atlassian.jira.notification.NotificationRecipient;
import com.atlassian.mail.queue.MailQueueItem;

public class MailServlet extends HttpServlet {
	private static final Logger log = LoggerFactory.getLogger(MailServlet.class);

	private final IssueMailQueueItemFactory issueMailQueueItemFactory;
	private final IssueEventManager issueEventManager;

	public MailServlet(IssueMailQueueItemFactory issueMailQueueItemFactory, IssueEventManager issueEventManager) {
		super();
		this.issueMailQueueItemFactory = issueMailQueueItemFactory;
		this.issueEventManager = issueEventManager;
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		resp.setContentType("text/html");

		String receiver = req.getParameter("receiver");
		String issueKey = req.getParameter("issue");
		User user = ComponentAccessor.getUserUtil().getUser(receiver);
		Issue issue = ComponentAccessor.getIssueManager().getIssueObject(issueKey);
		if (issue != null) {
			User loggedInUser = ComponentAccessor.getJiraAuthenticationContext().getLoggedInUser();
			Map<String, Object> context = new HashMap<String, Object>();
			context.put("sender", loggedInUser);
			context.put("baseurl", ComponentAccessor.getApplicationProperties().getString(APKeys.JIRA_BASEURL));
			if (user != null) {
				NotificationRecipient recipient = new NotificationRecipient(user);
				HashSet<NotificationRecipient> recipients = new HashSet<NotificationRecipient>();
				recipients.add(recipient);

				IssueEvent event = new IssueEvent(issue, context, loggedInUser, 10000L);

				MailQueueItem item = issueMailQueueItemFactory.getIssueMailQueueItem(event, 99L, recipients, "");
				ComponentAccessor.getMailQueue().addItem(item);

				resp.getWriter().write("<html><body>Mail sent to " + receiver + ", yay!</body></html>");
			} else {
				issueEventManager.dispatchEvent(10000L, issue, context, loggedInUser, true);

				resp.getWriter().write("<html><body>Mail sent via notification scheme, yay!</body></html>");
			}
		} else if (issue == null) {
			resp.getWriter().write("<html><body>Unknown issue:" + issueKey + ". Mail not sent!</body></html>");
		}
	}
}